function [CMean_m,Gamma,R,CFE]...
    =cvb(Data,K,no_reps,KMean,Alpha_0,m_0,v_0,a_0,b_0,mdisplay)


 

%Input variables
%Data--- D x G (data x gene)  matrix of expression values
%K---number of processes
%no_reps---number of iterations in the algorithm
%Alpha_0 ---1 x  K, initial   Dirichlet hyperparameter ,
%m_0,v_0,a_0,b_0----initial hyperparameters
%for example, m_0=0,  v_0=0.1,  a_0=20.0,  b_0=0.05  
%tol-- tolerance for free energy increase
%KMean----initializing the mean CMean_v, this is very critical for the
%algorithm,  usually using k-means to initialize
 %Copyright (c)  Yiming Ying, 2008%%%%



%Outputs
%CMean_m---G x K matrix, mean hyperparameters over means parameters
%Gamma---D x K matrix,  clustering posterior Dirichlet parameter
%R----parameter for posterior process choosing latent variable Z; 
%CFE--- free energy lower bound for evidence

%%%%%% the data is processed with mean zero and std 5. 

 


%Initialize variables

[no_samp,no_feat]=size(Data);

 
 
 
% 
% [Y C]=kmeans_cluster(Data,K);
% 
% KMEAN=C';

CMean_m=KMean;






CMean_v=v_0*ones(no_feat,K);


CVar_ga=a_0*ones(no_feat,K);


CVar_gb=b_0*ones(no_feat,K);






 

 

 


 
 


R=(1/K).*ones(no_samp,no_feat,K);


for rep=1:no_reps      %  main iteration loop
 
    %Display iteration number
    if mdisplay==1
        v=sprintf('CVB Iteration: %d',rep);
        disp(v);
    end


%%%% updates for  r_{dg,k}, note that r_{dg,k} needs to be normalized w.r.t. the
%%%% last index k




 
 
       
%%%%%%%% the exponent of   \log {\mathbb E}_{q\backslash dg }
%%%%%%%%%%%%%%%%%%\bigl[\log (\ga_k + \sum_{g'\neq g}Z_{dg',k})\bigr]




 TEMPR=R;   TZ=zeros(no_samp,no_feat,K);
 
      
 
        for k=1:K
        
               temp1=reshape(sum(TEMPR(:, :, k),2),no_samp,1);
        
               TEMP=repmat(Alpha_0(k),no_samp,no_feat)...
                   +repmat(temp1,1,no_feat)-TEMPR(:, :, k);
      
               temp2=reshape(sum(TEMPR(:, :, k).*(1-TEMPR(:, :, k)),2),no_samp,1);
        
               temp3=repmat(temp2,1,no_feat)-TEMPR(:, :, k).*(1-TEMPR(:, :, k));
        
               TZ(:,:,k)=TEMP./exp((temp3)./(2*(TEMP).^2));
          
        end
        
clear temp1 temp2 temp3 TEMPR;


%%% the expectatioin of log Gaussian distribution part %%%%%%%%%

temp_ab=CVar_ga.*CVar_gb;   %%%%%%%%%%%%%%%  a_{gk} b_{gk}

N=zeros(no_samp,no_feat,K);


for d=1:no_samp
         
           temp1=(repmat(Data(d,:)',1,K)-CMean_m).^2;
         
           temp2=-0.5.*temp_ab.*(temp1+1./CMean_v)...
               +0.5.*(psi(CVar_ga)+log(CVar_gb));
         
           N(d,:,:)=reshape(temp2,1,no_feat,K);

end
      
       clear temp1 temp2;
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% the update of R equals to  TZ.*exp(N)%%%%%%%%%%%%%%%%%%%%%%%%%%%5       
       
     R=TZ.*exp(N);
   
     for d=1:no_samp    
         
         temp=reshape(R(d,:,:),no_feat,K);
        
         R(d,:,:)=reshape(temp./repmat(sum(temp,2),1,K),1,no_feat,K);
        
     end
   
   
   clear TZ N;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


   
 

 
 


%updates for CMean_v---v_{gk}


CR_temp=reshape(sum(R,1),no_feat,K);  %%%%summation of R  over the first index


CMean_v=v_0+temp_ab.*CR_temp;


%updates for Mean_m----m_{gk}
 
     for k=1:K
    
         
     CMean_m(:,k)=(1./CMean_v(:,k)).*(v_0*m_0+ temp_ab(:,k).*(sum(R(:,:,k).*Data))');

    
     end




%updates for Var_ga and Var_gb------a_{gk},  b_{gk}


CVar_ga=a_0+0.5*reshape(sum(R,1),no_feat,K);

         for k=1:K  
  
             temp_sum=reshape(R(:,:,k),no_samp,no_feat).*((Data-repmat(CMean_m(:,k)',no_samp,1)).^2);
    
    
             temp_gb=1/b_0+0.5*(sum(temp_sum)')+0.5*(1./CMean_v(:,k)).*CR_temp(:,k); 
             %%%here we use the summation of R over the first index:  CR_temp
    
    
             CVar_gb(:,k)=1./temp_gb;
    
    
         end
         
       
         
    
 
         
       
         
                                                                
         
end


        CFE=CFenergy(Data,CMean_m,CMean_v,R,CVar_ga,CVar_gb,...
    Alpha_0,m_0,v_0,a_0,b_0);
 

 


 Gamma=reshape(sum(R,2),no_samp,K);
 
 
%  
% NGamma=Gamma./repmat(sum(Gamma,2),1,K);
% 
% figure;
% bar_plot(NGamma');

 


 