function [Mean_m,Alpha]=realvb(Data,K,no_reps,m_0,v_0,a_0,b_0,mdisplay)


 

%Input variables
%Data--- D x G (data x gene)  matrix of expression values
%K---number of processes
%no_reps---number of iterations in the algorithm
%Alpha_0 ---1 x  K, initial   Dirichlet hyperparameter 
%m_0,v_0,a_0,b_0----initial hyperparameters
%for example, m_0=0,  v_0=0.1,  a_0=20.0,  b_0=0.05  
%tol-- tolerance for free energy increase
%KMean----initializing the mean Mean_m, this is very critical

 
 



%Outputs
%Mean_m---G x K matrix, mean hyperparameters over means parameters
%Alpha---D x K matrix,  clustering posterior Dirichlet parameter 
%FE--- free energy lower bound for evidence

  

 


%Initialize variables

[ns,nf]=size(Data);

Alpha_0=0.2*ones(1,K);

 
 

[Y C]=kmeans_cluster(Data,K);

KMean=C';
 


Mean_m=KMean;



Mean_v=v_0*ones(nf,K);


Var_ga=a_0*ones(nf,K);


Var_gb=b_0*ones(nf,K);






 
 

 

Free_en=[];

tempfr1=-(2e+8);

tempfr2=tempfr1+tol+1;


while ((rep<=no_reps) & norm(tempfr2-tempfr1)>=tol)       %  main iteration loop
 
    %  main iteration loop
 
 
    if mdisplay==1
        v=sprintf('Real VB iteration: %d',rep);
        disp(v);
    end
    
 
   
%updates for r_{dg,k}=P(Z_dg=k| gth_d), i.e., the probability of expression
%value for gene g in sample d produced by process k





%Caculating the summation of Alpha

sa=sum(Alpha,2);

 

temp_tgth=psi(Alpha)-psi(repmat(sa,1,K));     %%%% definition of log(\tilde(\theta)_{dk})



temp_ab=Var_ga.*Var_gb;   %%%%%%%%%%%%%%%  a_{gk} b_{gk}




%%%% updates for  r_{dg,k}, note that r_{dg,k} needs to be normalized w.r.t. the
%%%% last index k

R=(1/K).*ones(no_samp,no_feat,K);

T=zeros(no_samp,no_feat,K);

 


       for d=1:no_samp
         temp1=(repmat(Data(d,:)',1,K)-Mean_m).^2;
         temp2=repmat(temp_tgth(d,:),no_feat,1)-0.5.*(temp_ab.*temp1-1./Mean_v)+0.5.*(psi(Var_ga)+log(Var_gb));
         T(d,:,:)=reshape(temp2,1,no_feat,K);
       end


       for d=1:no_samp    
        temp1=reshape(-T(d,:,:),no_feat,K);
        M=mean(temp1,2);
        temp2=exp(-temp1+repmat(M,1,K));         
        R(d,:,:)=reshape(temp2./repmat(sum(temp2,2),1,K),1,no_feat,K);
        end







   
%updates for Alpha----alpha_{dk}

%Alpha_0_exp=Alpha_0./sum(Alpha_0);  %average (expectation) of Alpha_0

Gamma=repmat(Alpha_0,no_samp,1)+reshape(sum(R,2),no_samp,K);

 
 


%updates for Mean_v---v_{gk}


R_temp=reshape(sum(R,1),no_feat,K);  %%%%summation of R  over the first index


Mean_v=v_0+R_temp;


%updates for Mean_m----m_{gk}



     for k=1:K
    
         
     Mean_m(:,k)=(1./Mean_v(:,k)).*(v_0*m_0+ (sum(R(:,:,k).*Data))');

    
     end




%updates for Var_ga and Var_gb------a_{gk},  b_{gk}


Var_ga=a_0+0.5*reshape(sum(R,1),no_feat,K);


SUMR1=reshape(sum(R,1),no_feat,K)+(1e-60);


         for k=1:K
            
             MEANR=reshape(sum(R(:,:,k).*Data),no_feat,1)./SUMR1(:,k);
             
             TERM1=sum(R(:,:,k).*((Data-repmat(MEANR',no_samp,1)).^2))';
             
             TERM2=((v_0.*SUMR1(:,k))./(v_0+SUMR1(:,k))).*((m_0-MEANR).^2);
              
    temp_gb=1/b_0+0.5.*(TERM1+TERM2);
    
%%%here we use the summation of R over the first index:  R_temp
    
    Var_gb(:,k)=1./temp_gb;
    
         end
         
                                                              
         
end
 
 

Alpha_norm=Alpha./repmat(sum(Alpha,2),1,K);

figure;
bar_plot(Alpha_norm');
 


 